--
-- ddma-xrc.vhd - example demand-mode DMA design
--
-- (C) Copyright Alpha Data Parallel Systems Ltd. 2001-2006
--
-- Example program for:
--
--   o ADM-XRC
--   o ADM-XRC-P
--
-- Demand mode DMA is used to read data from the host via PLX DMA channel 0,
-- into a FIFO. The data read from the FIFO and written back to the host via
-- PLX DMA channel 1, also in demand-mode.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.fifo.all;
use work.localbus.all;
use work.image_pack.all;

entity ddma is
    generic(
        la_top:          in        natural := 23);
    port(
        lclk:            in        std_logic;
        lreset_l:        in        std_logic;
        lwrite:          in        std_logic;
        lads_l:          in        std_logic;
        lblast_l:        in        std_logic;
        lbterm_l:        inout     std_logic;
        ld:              inout     std_logic_vector(31 downto 0);
        la:              in        std_logic_vector(la_top downto 2);
        lready_l:        inout     std_logic;
        lbe_l:           in        std_logic_vector(3 downto 0);
        fholda:          in        std_logic;
        ldreq_l:         out       std_logic_vector(1 downto 0);
        ldack_l:         in        std_logic_vector(1 downto 0));
end ddma;

architecture mixed of ddma is

    --
    -- This value is the number of words / free locations that the design attempts to aggregate
    -- in the FIFO before attempting a demand-mode DMA burst. This is intended to increase the
    -- efficiency of the local bus by increasing the average burst length for demand-mode DMA
    -- transfers.
    --

    constant threshold : natural := 0;--32;

    signal rst : std_logic;
    signal lads_i : std_logic;
    signal lblast_i : std_logic;
    signal lbe_iq_l : std_logic_vector(3 downto 0);
    signal lwrite_i : std_logic;
    signal ldack_i : std_logic_vector(1 downto 0);
    signal ldack_q : std_logic_vector(1 downto 0);
    signal ld_iq : std_logic_vector(31 downto 0);
    signal ld_o : std_logic_vector(31 downto 0);
    signal la_i : std_logic_vector(la_top downto 2);
    signal lready_o_l : std_logic;
    signal lready_oe_l : std_logic;
    signal lbterm_i : std_logic;
    signal lbterm_o_l : std_logic;
    signal lbterm_oe_l : std_logic;
    signal ldreq_o_l : std_logic_vector(1 downto 0);
    signal ld_oe_l : std_logic;
    
    signal la_q : std_logic_vector(la_top downto 2);

    signal dd_qlads : std_logic_vector(1 downto 0);
    signal ds_qlads : std_logic;
    signal ds_idle : std_logic;
    signal ds_decode : std_logic;
    signal ds_write : std_logic;
    signal ds_xfer : std_logic;
    signal dd_idle : std_logic_vector(1 downto 0);
    signal dd_idle_q : std_logic_vector(1 downto 0);
    signal dd_idle_qq : std_logic_vector(1 downto 0);
    signal ds_ready : std_logic;
    signal ds_stop : std_logic;

    signal fifo_e : std_logic;
    signal fifo_ne : std_logic;
    signal fifo_read : std_logic;
    signal fifo_radvt : std_logic;
    signal fifo_rev : std_logic;
    signal fifo_f : std_logic;
    signal fifo_nf : std_logic;
    signal fifo_write : std_logic;
    signal fifo_level : std_logic_vector(15 downto 0);
    signal fifo_q : std_logic_vector(31 downto 0);

    signal dd_req : std_logic_vector(1 downto 0);
    signal dd_burst : std_logic_vector(1 downto 0);

    signal write_icount : std_logic;
    signal icount : std_logic_vector(29 downto 0);
    signal icount0 : std_logic;
    signal icountn : std_logic;
    signal wrote_icount: std_logic;
    signal write_ocount : std_logic;
    signal ocount : std_logic_vector(29 downto 0);
    signal ocount0 : std_logic;
    signal ocountn : std_logic;
    signal wrote_ocount : std_logic;

    signal logic0, logic1 : std_logic;

    attribute iob : string;
    attribute iob of lbe_iq_l : signal is "true";
    attribute iob of ld_iq : signal is "true";

------------------------------------------------------	 
	 type STATE_TYPE is (Sidle, S0, S1, S2, S3);
	 signal current_state, next_state: STATE_TYPE;

	 signal ready : std_logic;
	 signal cntr : integer;
	 signal cntw : integer;
	 signal cntop : integer;

------------------------------------------------------
	 component input1
		port(
		clk : in std_logic;
		reset : in std_logic;
		inpixels : in std_logic_vector(31 downto 0);
		
		out1 : out out_slice;
		out2 : out out_slice;
		out3 : out out_slice;
		ready : out std_logic
		);
	end component; 

	component operators is
	port(
		in1 : in std_logic_vector(47 downto 0);-- 6 up pixels
		in2 : in std_logic_vector(47 downto 0);-- 6 mid pixels
		in3 : in std_logic_vector(47 downto 0);-- 6 down pixels
		oper : in std_logic_vector(oper_size downto 0);
		
		outpixels : out std_logic_vector(31 downto 0)
	);
	end component;

	signal res : std_logic_vector(31 downto 0);
	signal bout1 : out_slice;
	signal bout2 : out_slice;
	signal bout3 : out_slice;
	signal inreset : std_logic;
	signal inready : std_logic;
	
	signal oper : std_logic_vector(oper_size downto 0);
	
	signal finished : std_logic;
	
begin

	inst1 : input1 port map (lclk,inreset,fifo_q ,bout1,bout2,bout3,inready);
	inst2 : operators port map (bout1,bout2,bout3,oper,res);

	 porc_update : process(lclk, current_state)
    begin
		  if rst = '1' then
			cntr <=0;
			cntop <=0;
        elsif rising_edge(lclk) then
				current_state <= next_state;
				if current_state = Sidle and cntop = 4 then
					cntr <= 0;
				end if;
				if current_state = Sidle and icount < width4*height + oper_size32 and icount > width4*height-2 then
					cntop <= cntop+1;
					oper <= oper(95 downto 0) & fifo_q;
				elsif current_state = S0 or current_state = S1 then
					cntop <=0;
					cntr <= cntr+1;
				end if;
        end if;
    end process;


    state_update : process(current_state, rst, fifo_level, cntr, cntw)
    begin
		next_state<=current_state;
		inreset <= '0';
	   if rst = '1' then
			ready <='0';
		else
			case current_state is
				-- attente du chargement de l'image dans la ram
				when Sidle =>
					ready <='0';
					if fifo_level < width4*height then--my_stack_size then
						next_state <= Sidle;
					else 
						next_state <= S0;
						inreset <= '1';
					end if;
				-- attente du chargement des trois premieres lignes (input seulement)
				when S0 =>
					ready <='0';
					if cntr < 2*width4-2 then
						next_state <= S0;
					else 
						next_state <= S1;
					end if;
				when S1 =>
				-- traitement de l'image
					ready <='0';
					if cntr < width4*height then
						next_state <= S1;
					else 
						next_state <= S2;
					end if;
				-- attente du traitement des trois dernieres lignes (output seulement)
				when S2 =>
					ready <= '0';
					if cntw < width4*height then
						next_state <= S2;
					else 
						next_state <= S3;
					end if;
				when S3 =>
					ready <= '1';
					if fifo_level /= 0 then
						next_state <= S3;
					else 
						next_state <= Sidle;
					end if;
			end case;
		end if;	
	 end process;
    
    logic0 <= '0';
    logic1 <= '1';

    --
    -- Convert the inputs to active high.
    --

    rst        <= not lreset_l;
    lads_i     <= not lads_l;
    lblast_i   <= not lblast_l;
    lbterm_i   <= not lbterm_l;
    lwrite_i   <= lwrite;
    la_i       <= la(la_top downto 2);
    ldack_i    <= not ldack_l;

    --
    -- Register LD and LBE# (should be mapped into IOBs)
    --

    gen_lbe_iq_l : process(rst, lclk)
    begin
        if rst = '1' then
            lbe_iq_l <= (others => '0');
        elsif lclk'event and lclk = '1' then
            lbe_iq_l <= lbe_l;
        end if;
    end process;

    gen_ld_iq : process(rst, lclk)
    begin
        if rst = '1' then
            ld_iq <= (others => '0');
        elsif lclk'event and lclk = '1' then
				if current_state = S1 or current_state = S2 then
					ld_iq <= res;
				else
            ld_iq <= ld;
				end if;
        end if;
    end process;

    --
    -- Output active low signals.
    --

    ldreq_l <= ldreq_o_l;

    --
    -- Generate qualified versions of 'lads_l'.
    --
    -- For direct slave access, this is when 'lads_l' is asserted AND the FPGA
    -- is being addressed AND the FPGA is not the local bus master AND the
    -- cycle is not demand-mode DMA.
    --
    -- For demand-mode DMA access, this is when 'lads_l' is asserted AND the
    -- FPGA is being addressed and the FPGA is not the local bus master. The
    -- demand-mode DMA acknowledge signals are checked within the 'plxddsm'
    -- module.
    --

    ds_qlads <= lads_i and not la_i(la_top) and not fholda;
    gen_dd_qlads : for i in ldreq_l'range generate
        dd_qlads(i) <= lads_i and not la_i(la_top) and not fholda and ldack_i(i);
    end generate;

    --
    -- 'lbterm_l' should only be driven when the FPGA is addressed; otherwise
    -- float, because the control logic on the card might also drive it.
    --

    lbterm_l <= lbterm_o_l when lbterm_oe_l = '0' else 'Z';

    --
    -- 'lready_l' should only be driven when the FPGA is addressed; otherwise
    -- float because the control logic on the card might also drive it.
    --
 
    lready_l  <= lready_o_l when lready_oe_l = '0' else 'Z';

    --
    -- Drive the local data bus on a read.
    --
    
    ld <= ld_o when ld_oe_l = '0' else (others => 'Z');

    --
    -- Latch 'lad' on assertion of 'lads_l', in order to improve timing. There
    -- is no need to increment the address, because we don't allow bursting for
    -- register addresses, and DMA transfers that burst are expected to use
    -- constant local addressing.
    --

    latch_address : process(rst, lclk)
    begin
        if rst = '1' then
            la_q <= (others => '0');
            ldack_q <= (others => '0');
        elsif lclk'event and lclk = '1' then
            if lads_i = '1' then
                la_q <= la_i;
                ldack_q <= ldack_i;
            end if;
        end if;
    end process;

    --
    -- Generate 'ld_o', data to be output on local bus.
    --
    -- Depending on whether this is a demand-mode DMA burst or a normal direct
    -- slave burst, and the local bus address, we output one of
    --
    --   o FIFO data 'fifo_q'
    --   o 'icount' register value
    --   o 'ocount' register value
    --

    gen_ld_o : process(rst, lclk, current_state,cntw)
    begin
        if rst = '1' then
            ld_o <= (others => '0');
        elsif lclk'event and lclk = '1' then
					if current_state = Sidle then
						cntw <= 0;
					end if;
            if ldack_q(1) = '1' then
                ld_o <= fifo_q;
				 
				elsif la_q(2) = '1' and la_q(3) = '1' then
					ld_o <= "0000000000000000000000000000000" & ready;
				end if;
					if current_state = S1 or current_state = S2 then
						cntw <= cntw+1;
					end if;

--            else
--                if la_q(2) = '1' then
--                    ld_o <= ocount & ocountn & ocount0;
--                else
--                    ld_o <= icount & icountn & icount0;
--                end if;
            --end if;
        end if;
    end process;

    --
    -- This process implements the DMA transfer count registers and flags for
    -- each direction.
    --
    --    icount          number of words remaining in PCI-to-local DMA transfer
    --    icount0         '1' => 'icount' is zero
    --    icountn         '1' => 'icount' is less than or equal to 'threshold'
    --    ocount          number of words remaining in local-to-PCI DMA transfer
    --    ocount0         '1' => 'ocount' is zero
    --    ocountn         '1' => 'ocount' is less than or equal to 'threshold'
    --
    -- Prior to the host kicking off a demand-mode DMA transfer, the host must
    -- program the transfer count registers must be programmed with the length
    -- of the DMA transfer being performed.
    --

    gen_counts: process(rst, lclk, current_state, cntop)
    begin
        if rst = '1' then
            write_icount <= '0';
            icount  <= (others => '0');
            icountn <= '1';
            icount0 <= '1';
            wrote_icount <= '0';
            write_ocount <= '0';
            ocount  <= (others => '0');
            ocountn <= '1';
            ocount0 <= '1';
            wrote_ocount <= '0';
		  elsif current_state /= S0 and current_state /= S1 and current_state /= S2 then
		  
				
        if lclk'event and lclk = '1' then
            write_icount <= ds_xfer and ds_write and not la_q(2) and not OR_reduce(ldack_q);
            write_ocount <= ds_xfer and ds_write and la_q(2) and not OR_reduce(ldack_q);

            if write_icount = '1' then
                if lbe_iq_l(0) = '0' then
                    icount(5 downto 0) <= ld_iq(7 downto 2);
                end if;
                if lbe_iq_l(1) = '0' then 
                    icount(13 downto 6) <= ld_iq(15 downto 8);
                end if;
                if lbe_iq_l(2) = '0' then 
                    icount(21 downto 14) <= ld_iq(23 downto 16);
                end if;
                if lbe_iq_l(3) = '0' then 
                    icount(29 downto 22) <= ld_iq(31 downto 24);
                end if;
            else
                if fifo_write = '1' then
                    icount <= icount - 1;
                end if;
            end if;

            if write_ocount = '1' then
                if lbe_iq_l(0) = '0' then
                    ocount(5 downto 0) <= ld_iq(7 downto 2);
                end if;
                if lbe_iq_l(1) = '0' then
                    ocount(13 downto 6) <= ld_iq(15 downto 8);
                end if;
                if lbe_iq_l(2) = '0' then 
                    ocount(21 downto 14) <= ld_iq(23 downto 16);
                end if;
                if lbe_iq_l(3) = '0' then 
                    ocount(29 downto 22) <= ld_iq(31 downto 24);
                end if;
            else
                if fifo_read = '1' then
                    ocount <= ocount - 1;
                end if;
            end if;

            wrote_icount <= write_icount;
            if wrote_icount = '1' then
                if icount <= threshold then
                    icountn <= '1';
                else
                    icountn <= '0';
                end if;
                if icount = 0 then
                    icount0 <= '1';
                else
                    icount0 <= '0';
                end if;
            else
                if fifo_write = '1' then
                    if icount = (threshold + 1) then
                        icountn <= '1';
                    end if;
                    if icount = 1 then
                        icount0 <= '1';
                    end if;
                end if;
            end if;

            wrote_ocount <= write_ocount;
            if wrote_ocount = '1' then
                if ocount <= threshold then
                    ocountn <= '1';
                else
                    ocountn <= '0';
                end if;
                if ocount = 0 then
                    ocount0 <= '1';
                else
                    ocount0 <= '0';
                end if;
            else
                if fifo_read = '1' then
                    if ocount = (threshold + 1) then
                        ocountn <= '1';
                    end if;
                    if ocount = 1 then
                        ocount0 <= '1';
                    end if;
                end if;
            end if;
				end if;
        end if;
    end process;

    --
    -- Delay the 'idle' signals from each of the demand-mode DMA state machines
    -- by one and two cycles.
    --

    gen_dd_idle_qq : process(rst, lclk)
    begin
        if rst = '1' then
            dd_idle_q <= (others => '1');
            dd_idle_qq <= (others => '1');
        elsif lclk'event and lclk = '1' then
            for i in ldreq_l'range loop
               dd_idle_q(i) <= dd_idle(i);
               dd_idle_qq(i) <= dd_idle_q(i);
            end loop;
        end if;
    end process;

    --
    -- For the PCI->local direction, we wait until the FIFO has space to hold a
    -- decent-sized burst before requesting a demand-mode DMA burst. We need to
    -- pause if the FIFO becomes nearly-full.
    --
    -- The 'dd_idle_qq(0)' term takes care of a corner case. At the end of a demand-
    -- mode DMA local bus cycle, it may take a couple of clock cycles for the FIFO
    -- flags and level to be fully updated to reflect how much data has just been
    -- entered into the FIFO. Therefore, we force 'dd_req(0)' deasserted for two
    -- cycles at the end of each demand-mode DMA local bus cycle.
    --
    -- This could be optimized by only forcing 'dd_req(0)' deasserted when the FIFO
    -- level is known to be above a certain threshold, but for simplicity we do it
    -- unconditionally.
    --

    dd_req(0) <= '1';--'1' when icount0 = '0' and dd_idle_qq(0) = '1' and fifo_level <= (512 - threshold) else '0';
    dd_burst(0) <= '1';--not fifo_nf;

    --
    -- For the local->PCI direction, we request a demand-mode DMA burst only if the
    -- DMA transfer is not complete ('ocount' is nonzero) and either:
    --
    --      (i) the FIFO contains enough data for a decent-sized burst, or
    --
    --     (ii) we're near the end of the DMA transfer AND the FIFO level
    --          is equal to or greater than the remaining transfer count.
    --
    -- The 'dd_idle_q(1)' term takes care of a corner case. At the end of a demand-
    -- mode DMA local bus cycle, it may take an extra clock cycle for the FIFO flags
    -- and level to be fully updated to reflect how much data has just been read out
    -- of the FIFO. Therefore, we force 'dd_req(1)' deasserted for one cycle at the
    -- end of each demand-mode DMA local bus cycle.
    --
    -- This could be optimized by only forcing 'dd_req(1)' deasserted when the FIFO
    -- level is known to be below a certain threshold, but for simplicity we do it
    -- unconditionally.
    --

    dd_req(1) <= '1';--'1' when (dd_idle_q(1) = '1' and ocount0 = '0' and fifo_level >= threshold) or (dd_idle_q(1) = '1' and ocount0 = '0' and ocountn = '1' and fifo_level >= ocount(15 downto 0)) else '0';
    dd_burst(1) <= '1';--not fifo_ne;

    --
    -- Writing into the FIFO only happens when data is written on PLX DMA channel 0.
    -- Because the data and byte enables from the local bus are registered, the
    -- 'fifo_write' signal must also be registered.
    --

    gen_fifo_write : process(rst, lclk, current_state, ds_xfer, ds_write, ldack_q)
    begin
        if rst = '1' then
            fifo_write <= '0';
        elsif lclk'event and lclk = '1' then
				if current_state = S1 or current_state = S2 then
					fifo_write <= '1';
				else
					fifo_write <= ds_xfer and ds_write and ldack_q(0);
				end if;
        end if;
    end process;

    --
    -- Advancing the FIFO permanent read pointer only happens when data is
    -- read on PLX DMA channel 1.
    --
    read_proc : process(current_state, ds_xfer, ds_write, ldack_q, ds_decode, icount)
    begin	 
if current_state = S0 or current_state=S1 then
	fifo_read <= '1';
	fifo_radvt <= '1';
elsif current_state = Sidle and icount < width4*height + oper_size32 and icount > width4*height-2 then
	fifo_read <= '1';
	fifo_radvt <= '1';
	
else
---------------------------
    fifo_read <= ds_xfer and not ds_write and ldack_q(1);
	 fifo_radvt <= (ds_xfer or ds_decode) and ldack_q(1);
---------------------------
end if;
    end process;
    --
    -- Advance the FIFO temporary read pointer during a burst. Because we are
    -- pipelining 'ld_o' we require the 'ds_decode' term in order to clock data
    -- through to 'ld_o' at the beginning of a read burst.
    --

--    fifo_radvt <= (ds_xfer or ds_decode) and ldack_q(1);

    --
    -- We make the FIFO's temporary read pointer revert to its permanent
    -- read pointer at the end of a demand-mode DMA read local bus cycle.
    --

    fifo_rev <= dd_idle(1) and not dd_idle_q(1);

    --
    -- Generate 'ready' and 'stop' signals, which control the state machines
    -- that respond to local bus cycles.
    --

    ds_ready <= '1';                               -- We're always ready
    ds_stop <= not OR_reduce(ldack_q) or           -- Prevent bursting if not demand-mode DMA
               (ldack_q(0) and not dd_burst(0)) or -- Stop when demand-mode DMA and write FIFO is nearly full
               (ldack_q(1) and not dd_burst(1));   -- Stop when demand-mode DMA and read FIFO is nearly empty

    --
    -- Instantiate the state machines that control the LDREQ# lines.
    --

    gen_plxddsm : for i in ldreq_l'range generate
        U0 : plxddsm
            port map(
            clk => lclk,
            rst => rst,
            sr => logic0,
            qlads => dd_qlads(i),
            lblast => lblast_i,
            lbterm => lbterm_i,
            request => dd_req(i),
            ready => ds_ready, -- MUST be same as fed into 'plxdssm'
            stop => ds_stop,   -- MUST be same as fed into 'plxdssm'
            idle => dd_idle(i),
            ldreq_o_l => ldreq_o_l(i));
    end generate;

    --
    -- Instantiate the direct slave state machine; monitors the local bus for
    -- direct slave cycles and responds appropriately.
    --

    plxdssm0: plxdssm
        port map(
            clk => lclk,
            rst => rst,
            sr => logic0,
            qlads => ds_qlads,
            lblast => lblast_i,
            lbterm => lbterm_i,
            lwrite => lwrite_i,
            eld_oe => open,
            ld_oe_l => ld_oe_l,
            lready_o_l => lready_o_l,
            lready_oe_l => lready_oe_l,
            lbterm_o_l => lbterm_o_l,
            lbterm_oe_l => lbterm_oe_l,
            idle => ds_idle,
            transfer => ds_xfer,
            decode => ds_decode,
            write => ds_write,
            ready => ds_ready, -- MUST be same as fed into 'plxddsm'
            stop => ds_stop);  -- MUST be same as fed into 'plxddsm'

    --
    -- 'rpel' = 5 => nearly-empty is 5 or less words present the FIFO.
    -- 'wpfl' = 6 => nearly-full is 6 or less free locations remaining in the FIFO.
    --
    -- These values are chosen because of an erratum in the PCI9656, where under
    -- certain circumstances, the PCI9656 transfers up to 3 extra words of data after
    -- it has been directed to pause a demand-mode DMA transfer.
    --

    sfifop0 : sfifop
	generic map(
            width      => 32,
            order      => 14,
            rpel       => 0,--5,
            wpfl       => 0,--6,
            family     => fifo_family_virtex2,
            safety     => false)
        port map(
            rst        => rst,
            k          => lclk,
            wd         => ld_iq,
            wadv       => fifo_write,
            we         => open,
            wpf        => fifo_nf,
            wf         => fifo_f,
            wovf       => open,
            rq         => fifo_q,--temp_fifo_q,--fifo_q,
            radv       => fifo_read,
            radvt      => fifo_radvt,
            rrev       => fifo_rev,
            rflush     => logic0,
            re         => fifo_e,
            rf         => open,
            rpe        => fifo_ne,
            rudf       => open,
            rlevel     => fifo_level);

end mixed;
