library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.image_pack.all;



entity input1 is
port(
	clk : in std_logic;
	reset : in std_logic;
	inpixels : in std_logic_vector(31 downto 0);
	
	out1 : out out_slice;
	out2 : out out_slice;
	out3 : out out_slice;
	ready : out std_logic
);
end input1; 


architecture arch1 of input1 is

type STATE_TYPE is (S0, S1, s0a);

--signal buff : std_logic_vector(input_size downto 0);
signal m1 : input_slice;
signal m2 : input_slice;
signal m3 : input_slice;

--signal next_buff : std_logic_vector(input_size downto 0);
signal next_m1 : input_slice;
signal next_m2 : input_slice;
signal next_m3 : input_slice;

signal last_pix : pixel;
signal next_last_pix : pixel;

SIGNAL next_c_hor: INTEGER RANGE 0 TO 1024;
SIGNAL next_c_ver: INTEGER RANGE 0 TO 1024;

SIGNAL next_out1 : out_slice;
SIGNAL next_out2 : out_slice;
SIGNAL next_out3 : out_slice;
SIGNAL ready_s, next_ready : std_logic;

SIGNAL c_hor: INTEGER RANGE 0 TO 1024;
SIGNAL c_ver: INTEGER RANGE 0 TO 1024;

signal current_state, next_state: STATE_TYPE;

begin
	SYNC_PROC: process (CLK, RESET)
	begin

		if rising_edge(clk) then
				m1 <= next_m1;
				m2 <= next_m2;
				m3 <= next_m3;
				c_hor <= next_c_hor;
				c_ver <= next_c_ver;
				ready_s <= next_ready;
				current_state <= next_state;
				
				last_pix <= next_last_pix;
		end if;
	end process;

	state_machine: process (current_state, reset, inpixels, m1,m2,m3, c_hor, c_ver, ready_s)--,...tous les signaux a droite des assignations (sauf etats et constantes)
	begin
		next_c_hor <= c_hor;
		next_c_ver <= c_ver;		
		next_m1 <= m1;
		next_m2 <= m2;
		next_m3 <= m3;
		next_state <= current_state;
		next_out1 <= m1(input_slice_size downto input_slice_size-47);
		next_out2 <= m2(input_slice_size downto input_slice_size-47);
		next_out3 <= m3(input_slice_size downto input_slice_size-47);

		next_last_pix <= m3(input_slice_size-24 downto input_slice_size-31);


		next_ready <= ready_s;
		ready <= ready_s;
		
	
		case current_state is
			when s0 =>
				next_c_hor <= 0;
				next_c_ver <= 0;

				next_state <= s0;
				next_out1 <= (others => '0');
				next_out2 <= (others => '0');
				next_out3 <= (others => '0');
				next_ready <= '0';
				
				next_last_pix <= (others => '0');
				
				if RESET='1' then
					next_state <= S1;
				end if;

			when s0a =>	
				next_state <= s1;
			when s1 =>

					if c_ver=0 then
							next_m1 <= m1(input_slice_size-32 downto 0) & inpixels;
							next_m2 <= m2(input_slice_size-32 downto 0) & inpixels;
							next_m3 <= m3(input_slice_size-32 downto 0) & m2(input_slice_size downto input_slice_size-31);
					elsif c_ver=height then
							next_m1 <= m1(input_slice_size-32 downto 0) & m1(input_slice_size downto input_slice_size-31);
							next_m2 <= m2(input_slice_size-32 downto 0) & m1(input_slice_size downto input_slice_size-31);
							next_m3 <= m3(input_slice_size-32 downto 0) & m2(input_slice_size downto input_slice_size-31);
					else
							next_m1 <= m1(input_slice_size-32 downto 0) & inpixels;
							next_m2 <= m2(input_slice_size-32 downto 0) & m1(input_slice_size downto input_slice_size-31);
							next_m3 <= m3(input_slice_size-32 downto 0) & m2(input_slice_size downto input_slice_size-31);
					end if;


					if(c_hor = width4-1) then
						next_c_hor <= 0;
						next_c_ver <= c_ver + 1;
					else
						next_c_hor <= c_hor + 1;
					end if;

					next_state<=s1;
					
					if c_hor = 0 then
						out1 <= m1(input_slice_size downto input_slice_size-7) & m1(input_slice_size downto input_slice_size-39);
						out2 <= m2(input_slice_size downto input_slice_size-7) & m2(input_slice_size downto input_slice_size-39);
						out3 <= m3(input_slice_size downto input_slice_size-7) & m3(input_slice_size downto input_slice_size-39);
					elsif c_hor = width4-1 then
						out1 <= m2(7 downto 0) & m1(input_slice_size downto input_slice_size-31) & m1(input_slice_size-24 downto input_slice_size-31);
						out2 <= m3(7 downto 0) & m2(input_slice_size downto input_slice_size-31) & m2(input_slice_size-24 downto input_slice_size-31);
						out3 <= last_pix & m3(input_slice_size downto input_slice_size-31) & m3(input_slice_size-24 downto input_slice_size-31);
					else
						out1 <= m2(7 downto 0) & m1(input_slice_size downto input_slice_size-39);
						out2 <= m3(7 downto 0) & m2(input_slice_size downto input_slice_size-39);
						out3 <= last_pix & m3(input_slice_size downto input_slice_size-39);
					end if;				
					
		end case;
		if (c_hor = width4-1 and c_ver = height+1) then
			next_state <= S0;
			next_ready <= '1';
		end if;
	end process;
end arch1;
