library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.image_pack.all;

entity nonmax is
port(
	a00 : in pixel;
	a01 : in pixel;
	a02 : in pixel;
	a10 : in pixel;
	a11 : in pixel;
	a12 : in pixel;
	a20 : in pixel;
	a21 : in pixel;
	a22 : in pixel;
	
	thres : in pixel;
	
	res : out pixel
);
end nonmax;

architecture arch1 of nonmax is

signal b10 : std_logic;
signal b12 : std_logic;

signal b01 : std_logic;
signal b21 : std_logic;

signal b02 : std_logic;
signal b20 : std_logic;

signal b00 : std_logic;
signal b22 : std_logic;

signal s1, s2, s3, s4, s5 : std_logic;

signal grad : std_logic_vector(5 downto 0);
signal th : std_logic_vector(5 downto 0);

signal dir : std_logic_vector(1 downto 0);

begin 

	dir <= a11(7 downto 6);
	grad <= a11(5 downto 0);
	th <= thres(5 downto 0);
	
	b10 <= '1' when grad > a10(5 downto 0) else '0';
	b12 <= '1' when grad > a12(5 downto 0) else '0';
	
	b01 <= '1' when grad > a01(5 downto 0) else '0';
	b21 <= '1' when grad > a21(5 downto 0) else '0';
	
	b02 <= '1' when grad > a02(5 downto 0) else '0';
	b20 <= '1' when grad > a20(5 downto 0) else '0';

	b00 <= '1' when grad > a00(5 downto 0) else '0';
	b22 <= '1' when grad > a22(5 downto 0) else '0';


	s1 <= '1' when (b10='1' and b12='1' and dir="00") else '0';
	s2 <= '1' when (b01='1' and b21='1' and dir="01") else '0';
	s3 <= '1' when (b02='1' and b20='1' and dir="11") else '0';
	s4 <= '1' when (b00='1' and b22='1' and dir="10") else '0';
	
	s5 <= '1' when (s1='1' or s2='1' or s3='1' or s4='1') else '0'; 

	-- without thresholding
	--res <= x"ff" when (s5='1') else x"00";
	
	-- with thresholding
	res <= x"ff" when (s5='1' and grad>=th) else x"00";

-- convention :
-- | : 00
-- __ : 01
-- / : 10
-- \ : 11

end arch1;

