library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.image_pack.all;

entity operator is
port(
	in1 : in std_logic_vector(23 downto 0);-- 3 up pixels
	in2 : in std_logic_vector(23 downto 0);-- 3 mid pixels
	in3 : in std_logic_vector(23 downto 0);-- 3 down pixels
	oper : in std_logic_vector(oper_size downto 0);
		
	outpixel : out  pixel
);
end operator;

architecture arch1 of operator is

component gauss3x3 is
port(
	a00 : in pixel;
	a01 : in pixel;
	a02 : in pixel;
	a10 : in pixel;
	a11 : in pixel;
	a12 : in pixel;
	a20 : in pixel;
	a21 : in pixel;
	a22 : in pixel;
	
	res : out pixel
);
end component;

component grad3x3 is
port(
	a00 : in pixel;
	a01 : in pixel;
	a02 : in pixel;
	a10 : in pixel;
	a11 : in pixel;
	a12 : in pixel;
	a20 : in pixel;
	a21 : in pixel;
	a22 : in pixel;
	
	res : out pixel
);
end component;

component comp9
port(
	a00 : in pixel;
	a01 : in pixel;
	a02 : in pixel;
	a10 : in pixel;
	a11 : in pixel;
	a12 : in pixel;
	a20 : in pixel;
	a21 : in pixel;
	a22 : in pixel;

	choice : in std_logic_vector(1 downto 0); -- 00 : min , 01 : mean , 10 : max , 11 : max
	
	res : out pixel
);
end component;

component gradPM
port(
	a00 : in pixel;
	a01 : in pixel;
	a02 : in pixel;
	a10 : in pixel;
	a11 : in pixel;
	a12 : in pixel;
	a20 : in pixel;
	a21 : in pixel;
	a22 : in pixel;
	
	res : out pixel
);
end component;

component nonmax
port(
	a00 : in pixel;
	a01 : in pixel;
	a02 : in pixel;
	a10 : in pixel;
	a11 : in pixel;
	a12 : in pixel;
	a20 : in pixel;
	a21 : in pixel;
	a22 : in pixel;
	
	thres : in pixel;
	
	res : out pixel
);
end component;

signal buff_gauss : pixel;
signal buff_grad : pixel;
signal buff_med : pixel;
signal buff_gradPM : pixel;
signal buff_nonmax : pixel;

signal op_choice : pixel;
signal th_choice : pixel;
signal med_choice : std_logic_vector(1 downto 0);

begin

	op_choice<=oper(7 downto 0);
	th_choice<=oper(15 downto 8);
	med_choice<=oper(9 downto 8);	

	inst1 : gauss3x3 port map (in1(23 downto 16), in1(15 downto 8), in1(7 downto 0), in2(23 downto 16), in2(15 downto 8), in2(7 downto 0), in3(23 downto 16), in3(15 downto 8), in3(7 downto 0), buff_gauss);
	inst2 : grad3x3  port map (in1(23 downto 16), in1(15 downto 8), in1(7 downto 0), in2(23 downto 16), in2(15 downto 8), in2(7 downto 0), in3(23 downto 16), in3(15 downto 8), in3(7 downto 0), buff_grad);
	inst3 : comp9    port map (in1(23 downto 16), in1(15 downto 8), in1(7 downto 0), in2(23 downto 16), in2(15 downto 8), in2(7 downto 0), in3(23 downto 16), in3(15 downto 8), in3(7 downto 0), med_choice, buff_med);
	inst4 : gradPM    port map (in1(23 downto 16), in1(15 downto 8), in1(7 downto 0), in2(23 downto 16), in2(15 downto 8), in2(7 downto 0), in3(23 downto 16), in3(15 downto 8), in3(7 downto 0), buff_gradPM);
	inst5 : nonmax    port map (in1(23 downto 16), in1(15 downto 8), in1(7 downto 0), in2(23 downto 16), in2(15 downto 8), in2(7 downto 0), in3(23 downto 16), in3(15 downto 8), in3(7 downto 0), th_choice, buff_nonmax);
	
	selection : process(in1, in2, in3, op_choice, th_choice, med_choice,buff_gauss, buff_grad, buff_med, buff_gradPM, buff_nonmax)
	begin
		case op_choice is
			when x"00" =>
				outpixel <= in2(15 downto 8);
			when x"01" =>
				outpixel <= buff_gauss;
			when x"02" =>
				outpixel <= buff_med;
			when x"03" =>
				outpixel <= buff_grad;
			when x"04" =>
				outpixel <= buff_gradPM;
			when others =>
				outpixel <= buff_nonmax;
		end case;
	end process;
end arch1;