--
-- test_ddma.vhd - Testbench for entity 'ddma'
--
-- (C) Copyright Alpha Data 2004-2005
--
-- Example testbench for:
--
--   o ADM-XRC
--   o ADM-XRC-P
--   o ADM-XRC-II-Lite
--   o ADM-XRC-II
--   o ADM-XRC-4LX
--   o ADM-XRC-4SX
--
-- NOT SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library std;
use std.textio.all;

library plxsim;
use plxsim.plxsim.all;

entity test_ddma is
end test_ddma;

library work;
use work.image_pack.all;

architecture behav of test_ddma is

    constant period : time := 25 ns;

    constant priorities : integer_vector_t(0 to 2) := (2 => 1, others => 0);

	 --constant size : natural := 356*292;
    constant size : natural := 32*32;--(356/4)*292;--my_stack_size; --width4*4 * height*8; -- 1024; --4004;
	 constant chunk_size : natural := size*8;
	 constant op_size : natural := 4*4;

    signal lclk:               std_logic := '1';
    signal lreset_l:           std_logic := '0';
    signal lwrite:             std_logic := 'Z';
    signal lads_l:             std_logic := 'Z';
    signal lblast_l:           std_logic := 'Z';
    signal lbterm_l:           std_logic := 'Z';
    signal ld:                 std_logic_vector(31 downto 0) := (others => 'Z');
    signal la:                 std_logic_vector(31 downto 2) := (others => 'Z');
    signal lready_l:           std_logic := 'Z';
    signal lbe_l:              std_logic_vector(3 downto 0) := (others => 'Z');
    signal ldreq_l:            std_logic_vector(1 downto 0) := (others => 'Z');
    signal ldack_l:            std_logic_vector(1 downto 0);
    signal fholda:             std_logic := '0';
    signal lhold:              std_logic_vector(2 downto 0);
    signal lholda:             std_logic_vector(2 downto 0);

    signal ddma_in0, ddma_in1 : locbus_ddma_in_t;
    signal ddma_out0, ddma_out1 : locbus_ddma_out_t := init_locbus_ddma_out;
    signal bus_in, bus_in0, bus_in1 : locbus_in_t;
    signal bus_out, bus_out0, bus_out1 : locbus_out_t := init_locbus_out;

    signal logic0, logic1 : std_logic;

    procedure report_message(
        str : in string)
    is
        variable msg : line;
    begin
        write(msg, str);
        writeline(output, msg);
    end;

    procedure wait_cycles(
        constant n : in natural;
        signal   k : in std_logic)
    is begin
        for i in 0 to n - 1 loop
            wait until k'event and k = '1';
        end loop;
	wait for 1 ns;
    end;

    component ddma
        port(
            lclk          : in    std_logic;
            lreset_l      : in    std_logic; 
            lwrite        : in    std_logic;
            lads_l        : in    std_logic;
            lblast_l      : in    std_logic;
            lbterm_l      : inout std_logic;
            ld            : inout std_logic_vector(31 downto 0);
            la            : in    std_logic_vector(23 downto 2);
            lready_l      : inout std_logic; 
            lbe_l         : in    std_logic_vector(3 downto 0);
            ldack_l       : in    std_logic_vector(1 downto 0);
            ldreq_l       : out   std_logic_vector(1 downto 0);
            fholda        : in    std_logic);
    end component;

begin


    logic0 <= '0';
    logic1 <= '1';

    lclk <= not lclk after period / 2;

    lreset_l <= transport '1' after 50 ns;

    lads_l <= 'H';
    lblast_l <= 'H';
    lbterm_l <= 'H';
    lready_l <= 'H';
    lwrite <= 'H';

    stimulate_dd0 : process
		  variable dataop : byte_vector_t(0 to 127);
		  
		  
		  variable data : byte_vector_t(0 to chunk_size-1);
		  variable be : byte_enable_t(0 to chunk_size-1);
		  variable n : natural;
        variable val32 : std_logic_vector(31 downto 0);
        variable tag : natural;
        variable remaining : natural := size+op_size;
        variable chunk : natural;
		  
		  
        variable pat32, exp32 : std_logic_vector(31 downto 0);
        variable num_err : natural;
		  
    begin
	 
			---------------------------------------------------
        wait_cycles(4, lclk);

        be := (others => '1');

        -- Set inbound and outbound length registers
        data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(size+op_size, 32));
        data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(size+op_size, 32));

        plxsim_request_bus(true, bus_in, bus_out);

		  -- envoi de la taille de l'image ( nécéssaire pour DMA)
        plxsim_write(2, true, X"00000000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);  
        plxsim_request_bus(false, bus_in, bus_out);
		  
		  ---------------------------------------------------
        wait_cycles(20, lclk);

        be := (others => '1');
        tag := 0;
        val32 := X"beef0000";
		  remaining := size+op_size;
        while remaining /= 0 loop
            chunk := chunk_size;
            if chunk > remaining then
                chunk := remaining;
            end if;

            for i in 4 to chunk / 4 - 1 loop
                data(4 * i to 4 * (i + 1) - 1) := conv_byte_vector(val32 + tag + i);
            end loop;
				data(0 to 15):= conv_byte_vector(CONV_STD_LOGIC_VECTOR(0, 128));

            plxsim_wait_demand(bus_in0, ddma_in0);
            plxsim_request_bus(true, bus_in0, bus_out0);
            plxsim_write_const_demand(2, X"00000000", be(0 to chunk - 1), data(0 to chunk - 1), n, bus_in0, bus_out0, ddma_in0, ddma_out0);
            plxsim_request_bus(false, bus_in0, bus_out0);

            remaining := remaining - n;
            tag := tag + (n / 4);
				

        end loop;

        report_message("DDMA channel 0 process completed");

        --wait;
		  
		  
		  ----------------------------------------------------------------
        wait_cycles(400, lclk);
		  
		  --report_message("begin send back");
		  remaining := size;
        be := (others => '1');
        tag := 0;
        pat32 := X"beef0000";
        num_err := 0;
        while remaining /= 0 loop
            chunk := chunk_size;
            if chunk > remaining then
                chunk := remaining;
            end if;
				
	report_message("bla n : "& conv_string_hex(n));
            plxsim_wait_demand(bus_in1, ddma_in1);
            plxsim_request_bus(true, bus_in1, bus_out1);
            plxsim_read_const_demand(2, X"00000000", be(0 to chunk - 1), data(0 to chunk - 1), n, bus_in1, bus_out1, ddma_in1, ddma_out1);
            plxsim_request_bus(false, bus_in1, bus_out1);

            remaining := remaining - n;

            for i in 0 to (n / 4) -1 loop
                val32 := conv_std_logic_vector(data(4 * i to 4 * (i + 1) - 1))-op_size/4;
                exp32 := pat32 + tag;
                if exp32 /= val32 then
                    report_message("*** verification error: expected=0x" & conv_string_hex(exp32) &
                         " actual=0x" & conv_string_hex(val32));
                    num_err := num_err + 1;
                end if;
                tag := tag + 1;
            end loop;
        end loop;
		  
report_message("end");
	  
        report_message("Simulation of SIMPLE complete");
	
  --      report_message("DDMA channel 1 process completed with " & conv_string(num_err) & " data error(s)");

report_message("SECOND PASS");

    end process;

    UUT : ddma
        port map(
            lclk       => lclk,
            lreset_l   => lreset_l,
            lwrite     => lwrite,
            lads_l     => lads_l,
            lblast_l   => lblast_l,
            lbterm_l   => lbterm_l,
            ld         => ld,
            la         => la(23 downto 2),
            lready_l   => lready_l,
            lbe_l      => lbe_l,
            ldack_l    => ldack_l,
            ldreq_l    => ldreq_l,
            fholda     => fholda);
         
    agent_ds : locbus_agent_nonmux
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            la          => la,
            lbe_l       => lbe_l,
            lblast_l    => lblast_l,
            ld          => ld,
            lbterm_l    => lbterm_l,
            lready_l    => lready_l,
            lwrite      => lwrite,
            lhold       => lhold(2),
            lholda      => lholda(2),
            bus_in      => bus_in,
            bus_out     => bus_out);

    agent_dd0 : locbus_agent_nonmux
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            la          => la,
            lbe_l       => lbe_l,
            lblast_l    => lblast_l,
            ld          => ld,
            lbterm_l    => lbterm_l,
            lready_l    => lready_l,
            lwrite      => lwrite,
            lhold       => lhold(0),
            lholda      => lholda(0),
            bus_in      => bus_in0,
            bus_out     => bus_out0);

    agent_dd1 : locbus_agent_nonmux
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            la          => la,
            lbe_l       => lbe_l,
            lblast_l    => lblast_l,
            ld          => ld,
            lbterm_l    => lbterm_l,
            lready_l    => lready_l,
            lwrite      => lwrite,
            lhold       => lhold(1),
            lholda      => lholda(1),
            bus_in      => bus_in1,
            bus_out     => bus_out1);

    ddma_agent_0 : locbus_agent_ddma
        port map(
            ldreq_l     => ldreq_l(0),
            ldack_l     => ldack_l(0),
            dd_in       => ddma_in0,
            dd_out      => ddma_out0);

    ddma_agent_1 : locbus_agent_ddma
        port map(
            ldreq_l     => ldreq_l(1),
            ldack_l     => ldack_l(1),
            dd_in       => ddma_in1,
            dd_out      => ddma_out1);

    arb_0 : locbus_arb
        generic map(
            n_arb       => 3,
            priority    => priorities)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lhold       => lhold,
            lholda      => lholda);

    lbpcheck_0 : lbpcheck
        generic map(
            multiplexed => false,
            wide        => false)
        port map(
            lclk        => lclk,
            lreset_l    => lreset_l,
            lads_l      => lads_l,
            l64_l       => lads_l,
            la          => la,
            lad_lo      => ld,
            lad_hi      => ld,
            lbe_lo_l    => lbe_l,
            lbe_hi_l    => lbe_l,
            lwrite      => lwrite,
            lblast_l    => lblast_l,
            lready_l    => lready_l,
            lbterm_l    => lbterm_l);

end behav;
